@@ -301,7 +301,7 @@ class Ppos(Module):
         data = rlp.encode([rlp.encode(int(3000)), rlp.encode(typ), rlp.encode(data)])
         return send_obj_transaction(self, data, self.web3.penaltyAddress, pri_key, transaction_cfg)
 
-    def checkDuplicateSign(self, typ, check_address, block_number, from_address=None):
+    def checkDuplicateSign(self, typ, node_id, block_number, from_address=None):
         """
         Check if the node has been reported too much
         :param typ: Represents double sign type, 1:prepareBlock, 2: prepareVote, 3:viewChange
@@ -311,8 +311,7 @@ class Ppos(Module):
         :return:
         todo fill
         """
-        check_address = bech32_address_bytes(check_address)
-        data = rlp.encode([rlp.encode(int(3001)), rlp.encode(int(typ)), rlp.encode(check_address), rlp.encode(block_number)])
+        data = rlp.encode([rlp.encode(int(3001)), rlp.encode(int(typ)), rlp.encode(bytes.fromhex(node_id)), rlp.encode(block_number)])
         raw_data = call_obj(self, from_address, self.web3.penaltyAddress, data)
         receive = str(raw_data, encoding="ISO-8859-1")
         if receive == "":
@@ -406,8 +405,8 @@ class Ppos(Module):
 
     def getDelegateReward(self, from_address, node_ids=[]):
         node_id_bytes = [bytes.fromhex(node_id) for node_id in node_ids]
-        from_address = bech32_address_bytes(from_address)
-        data = [rlp.encode(int(5100)), rlp.encode(from_address), rlp.encode(node_id_bytes)]
+        tmp_from_address = bech32_address_bytes(from_address)
+        data = [rlp.encode(int(5100)), rlp.encode(tmp_from_address), rlp.encode(node_id_bytes)]
         data = rlp.encode(data)
         raw_data = call_obj(self, from_address, self.web3.delegateRewardAddress, data)
         receive = json.loads(str(raw_data, encoding="ISO-8859-1"))
